import copy

__author__ = 'modlab'

from rdkit import Chem
from rdkit.Chem import Descriptors
import collections

# TODO: define a function for each descriptor element and than successice function can be called to build up the
# descriptor vector


class DataExtractor(object):
    def __init__(self, sdf_file_path, num_shells, add_labute=False, add_nbo=False, shell_differences=False):
        self.sdf_file_path = sdf_file_path
        self.num_shells = num_shells
        self.descriptor_vectors = []
        self.add_labute = add_labute
        self.add_nbo = add_nbo
        self.shell_differences = shell_differences

    def load_molecules(self):
        self.suppl = Chem.SDMolSupplier(self.sdf_file_path, removeHs=False)

    def build_chargeshell_descriptor(self, atom, parent_mol, pc_name):
        """

        :param parent_mol:  rdkit.Chem.rdchem.Mol object
        :param atom:        rdkit.Chem.rdchem.Atom object
        :type parent_mol:   rdkit.Chem.rdchem.Mol
        :type atom:         rdkit.Chem.rdchem.Atom
        :return:
        """
        mol_charges = [float(charge) for charge in parent_mol.GetProp(pc_name).split()]

        # print mol_charges

        index_vector = []
        last_shell = []
        current_shell = []
        tmp_shell = []

        index_vector.append([atom.GetIdx()])
        contained_atoms = set()
        contained_atoms.add(atom)
        last_shell.append(atom)

        for shell in range(self.num_shells):
            for ls_atom in last_shell:
                for neighbour_atom in ls_atom.GetNeighbors():
                    if neighbour_atom.GetAtomicNum() != 1:
                        tmp_shell.append(neighbour_atom)
            # In the following lines the atoms need to be named differently from the input atom, otherwise code gets
            # messed up and I really do not understand why!
            # OK, I checked: Its a python 2.x "Bug" and the scope is leaked to outside. Unwanted but not fixable
            # Problem is fixed in python 3.x
            current_shell = [atm for atm in tmp_shell if atm not in contained_atoms]
            contained_atoms.update(current_shell)
            indices = [atm.GetIdx() for atm in current_shell]
            index_vector.append(indices)
            tmp_shell = []
            last_shell = current_shell

        # print index_vector

        charge_shell_vector = []
        charge_shell_vector.append(parent_mol.GetProp('ID'))
        charge_shell_vector.append(atom.GetIdx()+1)

        # TODO: SOM indices sollten pro Molekuel gespeichert und nicht bei jedem Atom neu generiert werden
        parent_som_indices = self.get_som_indices(parent_mol)
        if atom.GetIdx()+1 in parent_som_indices:
            charge_shell_vector.append(1)
        else:
            charge_shell_vector.append(0)

        # This is set, so that the first element is the charge of atom
        charge_last_shell = 0

        for shell in index_vector:
            if isinstance(shell, collections.Iterable):
                charges = [mol_charges[idx] for idx in shell]
                shell_charge = sum(charges) / len(charges)
                # shell_charge = sum(charges)
                if self.shell_differences:
                    charge_shell_vector.append(shell_charge - charge_last_shell)
                else:
                    charge_shell_vector.append(shell_charge)
            else:
                shell_charge = mol_charges[shell]
                if self.shell_differences:
                    charge_shell_vector.append(shell_charge - charge_last_shell)
                else:
                    charge_shell_vector.append(shell_charge)
            charge_last_shell = shell_charge

        assert len(charge_shell_vector) == self.num_shells+4

        if self.add_labute:
            Chem.MolSurf.pyLabuteASA(parent_mol)
            surf_areas = parent_mol._labuteContribs[1:]
            charge_shell_vector.append(surf_areas[atom.GetIdx()])
        if self.add_nbo:
            occupations = parent_mol.GetProp('p orbital occupations').split('\n')
            occ_px, occ_py, occ_pz = occupations[atom.GetIdx()].split()[1:]
            charge_shell_vector.append(float(occ_px))
            charge_shell_vector.append(float(occ_py))
            charge_shell_vector.append(float(occ_pz))
            energies = parent_mol.GetProp('p orbital energies').split('\n')
            en_px, en_py, en_pz = energies[atom.GetIdx()].split()[1:]
            charge_shell_vector.append(float(en_px))
            charge_shell_vector.append(float(en_py))
            charge_shell_vector.append(float(en_pz))
        print charge_shell_vector
        return charge_shell_vector

    def build_graph_chargeshell_descriptor(self, atom, parent_mol, pc_name, includeH=True):
        """
        Only a block as such is sorted intrinsically ...
        :param parent_mol:  rdkit.Chem.rdchem.Mol object
        :param atom:        rdkit.Chem.rdchem.Atom object
        :type parent_mol:   rdkit.Chem.rdchem.Mol
        :type atom:         rdkit.Chem.rdchem.Atom
        :return:
        """
        mol_charges = [float(charge) for charge in parent_mol.GetProp(pc_name).split()]

        index_vector = []
        last_shell = []
        current_shell = []
        tmp_shell = []
        block = [] # Corresponds to a block of atoms which then have to be sorted, could be sourced to a own class

        index_vector.append(atom.GetIdx())
        contained_atoms = set()
        contained_atoms.add(atom)
        last_shell.append(atom)

        for shell in range(self.num_shells):
            for ls_atom in last_shell:
                # loop trhough shell and create next shell
                # create and sort subshells / blocksb
                for neighbour_atom in ls_atom.GetNeighbors():
                    if neighbour_atom not in contained_atoms:
                        block.append(neighbour_atom)
                self._process_block(block, mol_charges)
                tmp_shell = tmp_shell + block
                block = []
            current_shell = tmp_shell
            contained_atoms.update(current_shell)
            indices = [atm.GetIdx() for atm in current_shell]
            index_vector += indices
            tmp_shell = []
            last_shell = current_shell

        charge_shell_vector = []
        charge_shell_vector.append(parent_mol.GetProp('ID'))
        charge_shell_vector.append(atom.GetIdx()+1)

        # TODO: SOM indices sollten pro Molekuel gespeichert und nicht bei jedem Atom neu generiert werden
        parent_som_indices = self.get_som_indices(parent_mol)
        if atom.GetIdx()+1 in parent_som_indices:
            charge_shell_vector.append(1)
        else:
            charge_shell_vector.append(0)

        # This is set, so that the first element is the charge of atom
        charge_last_shell = 0

        for idx in index_vector:
            if idx >= 0:
                charge_shell_vector.append(mol_charges[idx])
            else:
                charge_shell_vector.append(0.0)

        # assert len(charge_shell_vector) == 85 + 3 # TODO: NON GENERIC WATCH OUT

        if self.add_labute:
            Chem.MolSurf.pyLabuteASA(parent_mol)
            surf_areas = parent_mol._labuteContribs[1:]
            charge_shell_vector.append(surf_areas[atom.GetIdx()])
        if self.add_nbo:
            occupations = parent_mol.GetProp('p orbital occupations').split('\n')
            occ_px, occ_py, occ_pz = occupations[atom.GetIdx()].split()[1:]
            charge_shell_vector.append(float(occ_px))
            charge_shell_vector.append(float(occ_py))
            charge_shell_vector.append(float(occ_pz))
            energies = parent_mol.GetProp('p orbital energies').split('\n')
            en_px, en_py, en_pz = energies[atom.GetIdx()].split()[1:]
            charge_shell_vector.append(float(en_px))
            charge_shell_vector.append(float(en_py))
            charge_shell_vector.append(float(en_pz))
        print charge_shell_vector
        return charge_shell_vector


    def get_som_indices(self, molecule):
        prop_names = molecule.GetPropNames()
        som_prop_names = [name for name in prop_names if 'SOM' in name]
        som_indices = set()
        for spn in som_prop_names:
            indices = molecule.GetProp(spn)
            for index in indices.split():
                som_indices.add(int(index))
        return som_indices
        
    def create_decriptor_vectors(self, pc_name):
        for mol in self.suppl:
            for atom in mol.GetAtoms():
                if atom.GetAtomicNum() != 1:
                    self.descriptor_vectors.append(self.build_graph_chargeshell_descriptor(atom, mol, pc_name))

    def _process_block(self, block, mol_charges):
        block.sort(key=lambda atm: mol_charges[atm.GetIdx()])
        assert len(block) <= 4, 'There is a atom with more than 4 neighbours!\n{}'.format(block)
        while len(block) < 4:
            block.append(DummyA())


# testfile = '/Volumes/Platte1/Project/Calculations/s-oxidation/charge_files/S-oxidation.03.3D.unique_opt.sdf'
# testfile2 = '/Volumes/Platte1/Project/Calculations/XenoSite/charge_files/HLM_DFTB_OPT2_with_charges.sdf'

# de = DataExtractor(testfile2, 4)
# de.load_molecules()
# de.create_decriptor_vectors('npa')
# with open('/Volumes/Platte1/Project/Calculations/XenoSite/charge_files/data_vectors.txt', 'w') as f:
#     for dv in de.descriptors:
#         tmp_string1 = [str(i) for i in dv[:3]]
#         first_elements = '_'.join(tmp_string1)
#         tmp_string2 = '{: f}\t{: f}\t{: f}\t{: f}\t{: f}'.format(*dv[3:])
#         # data = '\t'.join(tmp_string2)
#         print first_elements + '\t' + tmp_string2
#         f.write(first_elements + '\t' + tmp_string2 + '\n')

class DummyA(object):
    def __init__(self):
        self.charge = 0

    def GetNeighbors(self):
        return []

    def GetIdx(self):
        return -1