from DescriptorElement import DescriptorElement
import numpy as np

__author__ = 'modlab'


class DistanceCom(DescriptorElement):
    description = 'Calculates distance to geometrical center of mass'

    def __init__(self, **options):
        super(DistanceCom, self).__init__(**options)

    def calculate_elements(self, atom):
        parent_mol = atom.GetOwningMol()
        nominator = np.zeros(3)
        for atm in parent_mol.GetAtoms():
            nominator += atm.GetMass() * np.array(parent_mol.GetConformer().GetAtomPosition(atm.GetIdx()))
        denominator = sum([atm.GetMass() for atm in parent_mol.GetAtoms()])
        com = nominator/denominator
        distance = np.linalg.norm(parent_mol.GetConformer().GetAtomPosition(atom.GetIdx()) - com)
        return [distance, (distance/denominator * 100)]
    # The latter shall assure that a for a heavy molecule the short distance is worse than for a very small molecue
    # This is just some intuition that smaller molecules can easier orient in pockets wheres larger (heavier) molecules
    # it is more important to be rather far away from the COM the be closer to an active site

