#!/Users/modlab/anaconda2/envs/rdkit-env/bin/python

import sys
import argparse

sys.path.extend(['PYTHON_PATH/site-packages/'])

from DescriptorCreator import AtomDescriptorCreator
from PartialChargeDict import PartialChargeDict as PCD


__author__ = 'modlab'

disclaimer = """
Program calculates atomic descriptors based on atomic properties in SD file.

WARNING!\n
This program is in development stage. There is absolutely no
warranty for correctness of the descriptor implementations!!!
The program is currently extended and tested. Results
might change with future versions.\n"""


def check_chargetype(pc_name):
    if pc_name in PCD.defined_partial_charges:
        return
    else:
        raise Exception('pc_name <{}> not defined, choose one of: {}'.format(pc_name, PCD.defined_partial_charges))


parser = argparse.ArgumentParser()
parser.description = disclaimer
# subparsers = parser.add_subparsers(help='sub-command help', title='Subcommands', description='valid subcommands')

# subparsers2 = parser.add_subparsers(help='sub-command help', title='Subcommands2', description='valid subcommands',
#                                    dest='rdf')
group = parser.add_argument_group('Required arguments')
group.add_argument('-inp', '--input', required=True,  metavar=('input.sdf'),
                    help='Input SD file, REQUIRED')
group.add_argument('-out', '--output', required=True, metavar=('output.sdf'),
                    help='Output SD file, REQUIRED')

parser.add_argument('-sa', '--shell_average', nargs=2, metavar=('n_shells', 'pc_name'),
                    help='Charge averaged per topological shell')
parser.add_argument('-sg', '--sorted_graph', nargs=2, metavar=('n_shells', 'pc_name'),
                    help='Sorted shells based on molecular graph')

parser.add_argument('-gacf', '--graph_autocorrelation_function', nargs=1, metavar=('pc_name'),
                    help='Atomic autcorrelation-type descriptor based on molecular graph')
parser.add_argument('-acf', '--autocorrelation_function', nargs=1, metavar=('pc_name'),
                    help='Atom-centered real-space autocorrelation function. For further parameters see: Subcommands')

parser.add_argument('-rdf', '--radial_distribution_function', nargs=1,  metavar=('pc_name'),
                    help='Atom-centered real-space radial distribution function. '
                         'For further parameters see: Subcommands')
parser.add_argument('-sdf_id', '--output_sdf_descriptor_identifier', metavar=('sdf_id'), default='Descriptor',
                    type=str, help='Name of the SD file property where descriptor is printed. Default = Descriptor')
# NOte : if nargs is used, argparse assumes that the arguemnts are stored in a list

group2 = parser.add_argument_group('Additional options for AC descriptor\n'
                                   'Note that the AC descriptor will only be created if the -acf flags is specified')
group2.add_argument('-acf_rmin', type=float, default=1.0, help='Minimum radius. Default = 1.0.')
group2.add_argument('-acf_rmax', type=float, default=10.0, help='Maximum radius. Default = 10.0.')
group2.add_argument('-acf_no_h', type=bool, default=False, help='If H-atoms should be ignored. Default = False.')
group2.add_argument('-acf_step_size', type=float, default=0.2, help='Step size. Default = 0.2.')
group2.add_argument('-acf_split_pos_neg', type=bool, default=True, help='Split positive and negative summands. Doubles '
                                                                        'descriptor length. Default=True')
group2.add_argument('-acf_variant2', type=bool, default=False, help='Use variant 2 where shell charges are correlated'
                                                                   'to query atom charge. Default=False')

group3 = parser.add_argument_group('Additional options for RDF descriptor\n'
                                   'Note that the RDF descriptor will only be created if the -rdf flag is specified')
group3.add_argument('-rdf_beta', type=float, default=20.0, help='Width parameter of the gaussian distance function. Default = 20.0')
group3.add_argument('-rdf_scale', type=float, default=10.0, help='Scaling factor. Default = 10.0')
group3.add_argument('-rdf_rmin', type=float, default=1.0, help='Minimum radius. Default = 1.0')
group3.add_argument('-rdf_rmax', type=float, default=11.0, help='Maximum radius. Default = 11.0')
group3.add_argument('-rdf_step_size', type=float, default=0.2, help='Step size to increase R. Default = 0.2')
group3.add_argument('-rdf_no_h', type=bool, default=False, help='If H-atoms should be exclluded. Default = False.')

group4 = parser.add_argument_group('Additional options for GAC descriptor\n'
                                   'Note that the GAC descriptor will only be created if the -gacf flag is specified')
group4.add_argument('-gacf_rmin', type=int, default=1, help='Minimum number of bonds. Default = 1')
group4.add_argument('-gacf_rmax', type=int, default=6, help='Maximum number of bonds. Default = 6')
group4.add_argument('-gacf_no_h', type=bool, default=False, help='If H-atoms should be excluded. Default = False.')

args = parser.parse_args()
# print args

# # Check for dependencies
# The dependency checking does not really work well because the defaults are given, which I like to keep the defaults
# obvious. Thus checking for those options will always be True. I "solved" the problem by adding a comment.
# if args.acf_rmin or args.acf_rmax is not None:
#     if not args.average_charge_shell:
#         msg = 'Options for AC descriptor but no AC descriptor specified'
#         raise argparse.ArgumentTypeError(msg)
#
# if args.rdf_beta or args.rdf_scale or args.rdf_rmin or args.rdf_rmax or args.rdf_stepsize is not None:
#     if not args.radial_distribution_function:
#         msg = 'Options for RDF descriptor but not RDF descriptor specified'
#         raise argparse.ArgumentTypeError(msg)


dc = AtomDescriptorCreator(args.input)

if args.shell_average:
    acg = args.shell_average
    check_chargetype(acg[1])
    dc.create_property('ChargeShell', n_shells=int(acg[0]), charge_type=acg[1])
if args.sorted_graph:
    gcs = args.sorted_graph
    check_chargetype(gcs[1])
    dc.create_property('GraphChargeShell', n_shells=int(gcs[0]), charge_type=gcs[1])
if args.autocorrelation_function:
    ac = args.autocorrelation_function
    check_chargetype(ac[0])
    dc.create_property('ChargeAutocorrelation', charge_type=ac[0], rmin=args.acf_rmin, rmax=args.acf_rmax,
                       no_h=args.acf_no_h, split_pos_neg=args.acf_split_pos_neg, variant2=args.acf_variant2)
if args.graph_autocorrelation_function:
    gac = args.graph_autocorrelation_function
    check_chargetype(gac[0])
    dc.create_property('ChargeGraphAutocorrelation', charge_type=gac[0], rmin=args.gacf_rmin, rmax=args.gacf_rmax,
                       no_h=args.gacf_no_h)
if args.radial_distribution_function:
    rdf = args.radial_distribution_function
    check_chargetype(rdf[0])
    dc.create_property('ChargeRDF', charge_type=rdf[0], beta=args.rdf_beta, scaling=args.rdf_scale,
                    rmin=args.rdf_rmin, rmax=args.rdf_rmax, step_size=args.rdf_step_size, no_h=args.rdf_no_h)

if args.output_sdf_descriptor_identifier:
    name = args.output_sdf_descriptor_identifier
else:
    name = 'Descriptor'

print('Beginning descriptor calculation')

dc.create_descriptor_vectors()
dc.write_sdf(args.output, descriptor_id=name)

print('Descriptor calculation done\n\n')

