from DescriptorCreator_ETH_pKBHX.DescriptorCreator import AtomDescriptorCreator
from sklearn.externals import joblib
from rdkit import Chem
import numpy as np

# This script will only work if the acceptor and donor models have been trained and stored!

def acc_predict(sdf,descriptor_elements,acc_regressor):
   # create the desciptor vectors
   adc = AtomDescriptorCreator(sdf)
   for des in descriptor_elements:
        adc.create_property(des[0], **des[1]) 
   adc.create_descriptor_vectors()
   # initialize the regression model
   accreg = joblib.load(acc_regressor)
   # predict
   pred = accreg.predict(adc.X,return_std = True)
   pred = np.asarray(pred)
   # print
   suppl = Chem.SDMolSupplier(sdf)
   counter = 0 
   for mol in suppl:
       molid = mol.GetProp('Mol_ID')
       atomindexlist = mol.GetProp('HBA_atom_index').split('\n')
       atomindexlist = [int(x)-1 for x in atomindexlist]
       print "Molecule: ", molid
       for index in atomindexlist:
           for atom in mol.GetAtoms():
               if atom.GetIdx() == index:
                   print "Acceptor atom", atom.GetSymbol(), str(atom.GetIdx() + 1), " : "
                   dGkjmol = pred[0,counter]
                   stdevkjmol = pred[1,counter]
                   print "predicted Delta G for this ", atom.GetSymbol()    , ": %.2f  +/- %.2f kJ/mol" %(dGkjmol,np.abs(stdevkjmol))
                   counter += 1

def don_predict(sdf,descriptor_elements,regressor_don):
   # create the desciptor vectors
   adc = AtomDescriptorCreator(sdf)
   for des in descriptor_elements:
        adc.create_property(des[0], **des[1]) 
   adc.create_descriptor_vectors_hbd()
   # initialize the regression model
   regdon = joblib.load(regressor_don)
   # predict
   pred = regdon.predict(adc.X,return_std = True)
   pred = np.asarray(pred)
   # print
   suppl = Chem.SDMolSupplier(sdf)
   counter = 0 
   for mol in suppl:
       molid = mol.GetProp('Mol_ID')
       atomindexlist = mol.GetProp('HBD_atom_index').split('\n')
       atomindexlist = [int(x)-1 for x in atomindexlist]
       print "Molecule: ", molid
       for index in atomindexlist:
           for atom in mol.GetAtoms():
               if atom.GetIdx() == index:
                   print "Donor atom", atom.GetSymbol(), str(atom.GetIdx() + 1), " : "
                   dGkjmol = pred[0,counter]
                   stdevkjmol = pred[1,counter]
                   print "predicted Delta G for this ", atom.GetSymbol()    , ": %.2f  +/- %.2f kJ/mol" %(dGkjmol,np.abs(stdevkjmol))
                   counter += 1

def __main__():
# Acceptors:
# definition of acceptor test molecule
   acc_testmol = 'testmolecules/testmolecule_acetone.sdf'
# definition of the final descriptor combination
   d1 =('GraphChargeShell', {'charge_type': 'cm5', 'n_shells': 3, 'use_cip_sort'    : True})
   d2 =('ChargeAutocorrelation', {'charge_type': 'cm5', 'rmin': 1, 'rmax': 10,'step_size': 0.2})
   d3 =('ChargeShell', {'charge_type': 'cm5', 'n_shells': 7})
   descriptor_elements = [d1, d2, d3]
# define the regressor to load
   acc_regressor = 'regressors/acceptor_model.pkl'
# call the predictor function
   acc_predict(acc_testmol,descriptor_elements,acc_regressor)
# Donors   :
# definition of donor test molecule
   don_testmol = 'testmolecules/testmolecule_5-oh-flavone.sdf'
# definition of the final descriptor combination
   d1 =('GraphChargeShell', {'charge_type': 'cm5', 'n_shells': 2, 'use_cip_sort'    : True})
   d2 =('ChargeAutocorrelation', {'charge_type': 'cm5', 'rmin': 1, 'rmax': 10,'step_size': 0.2})
   d3 =('ChargeShell', {'charge_type': 'cm5', 'n_shells': 7})
   descriptor_elements = [d1, d2, d3]
# define the regressor to load
   don_regressor = 'regressors/donor_model.pkl'
# call the predictor function
   don_predict(don_testmol,descriptor_elements,don_regressor)


__main__()
